@@ -1,21 +1,5 @@
-/*
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
- *
- * The Original Code is Copyright (C) Blender Foundation
- * All rights reserved.
- */
+/* SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright Blender Foundation. All rights reserved. */
 
 /** \file
  * \ingroup bke
@@ -78,13 +62,12 @@ typedef struct SelfColDetectData {
  * Collision modifier code start
  ***********************************/
 
-/* step is limited from 0 (frame start position) to 1 (frame end position) */
 void collision_move_object(CollisionModifierData *collmd,
                            const float step,
                            const float prevstep,
                            const bool moving_bvh)
 {
-  unsigned int i = 0;
+  uint i = 0;
 
   /* the collider doesn't move this frame */
   if (collmd->is_static) {
@@ -1233,10 +1216,7 @@ static void hair_collision(void *__restrict userdata,
   }
 }
 
-static void add_collision_object(ListBase *relations,
-                                 Object *ob,
-                                 int level,
-                                 unsigned int modifier_type)
+static void add_collision_object(ListBase *relations, Object *ob, int level, uint modifier_type)
 {
   /* only get objects with collision modifier */
   ModifierData *cmd = BKE_modifiers_findby_type(ob, modifier_type);
@@ -1261,15 +1241,13 @@ static void add_collision_object(ListBase *relations,
   }
 }
 
-/* Create list of collision relations in the collection or entire scene.
- * This is used by the depsgraph to build relations, as well as faster
- * lookup of colliders during evaluation. */
 ListBase *BKE_collision_relations_create(Depsgraph *depsgraph,
                                          Collection *collection,
-                                         unsigned int modifier_type)
+                                         uint modifier_type)
 {
+  const Scene *scene = DEG_get_input_scene(depsgraph);
   ViewLayer *view_layer = DEG_get_input_view_layer(depsgraph);
-  Base *base = BKE_collection_or_layer_objects(view_layer, collection);
+  Base *base = BKE_collection_or_layer_objects(scene, view_layer, collection);
   const bool for_render = (DEG_get_mode(depsgraph) == DAG_EVAL_RENDER);
   const int base_flag = (for_render) ? BASE_ENABLED_RENDER : BASE_ENABLED_VIEWPORT;
 
@@ -1292,13 +1270,11 @@ void BKE_collision_relations_free(ListBase *relations)
   }
 }
 
-/* Create effective list of colliders from relations built beforehand.
- * Self will be excluded. */
 Object **BKE_collision_objects_create(Depsgraph *depsgraph,
                                       Object *self,
                                       Collection *collection,
-                                      unsigned int *numcollobj,
-                                      unsigned int modifier_type)
+                                      uint *numcollobj,
+                                      uint modifier_type)
 {
   ListBase *relations = DEG_get_collision_relations(depsgraph, collection, modifier_type);
 
@@ -1341,8 +1317,6 @@ void BKE_collision_objects_free(Object **objects)
   }
 }
 
-/* Create effective list of colliders from relations built beforehand.
- * Self will be excluded. */
 ListBase *BKE_collider_cache_create(Depsgraph *depsgraph, Object *self, Collection *collection)
 {
   ListBase *relations = DEG_get_collision_relations(
@@ -1572,7 +1546,7 @@ int cloth_bvh_collision(
   ClothVertex *verts = NULL;
   int ret = 0, ret2 = 0;
   Object **collobjs = NULL;
-  unsigned int numcollobj = 0;
+  uint numcollobj = 0;
   uint *coll_counts_obj = NULL;
   BVHTreeOverlap **overlap_obj = NULL;
   uint coll_count_self = 0;
